(function($){
    var cssPrefix="-webkit-";
    var picSlider =function(opts){
        this._options = $.extend({},this.options,opts);
        this._create();
    };

    picSlider.prototype={

        options: {

            /**
             * @property {Boolean} [loop=false] 是否连续滑动
             */
            loop: false,

            /**
             * @property {Number} [speed=400] 动画执行速度
             * @namespace options
             */
            speed: 400,

            /**
             * @property {Number} [index=0] 初始位置
             * @namespace options
             */
            index: 0,

            /**
             * @property {Object} [selector={container:'.ui-slider-group'}] 内部结构选择器定义
             * @namespace options
             */
            selector: {
                container: '.cui-pic-slider-container'    // 容器的选择器
            },
            needSliderDots:true,
            needSliderButton:false,
            content:[]
        },
        eventNs:"cui-pic-slider",
        template: '<div class="cui-pic-slider-group">' +
            '<%var i,len=content.length;for(i=0;i<len;i++){%>' +
            '<div class="cui-pic-slider-item">' +
            '<a href="<%= content[i].href %>"><img src="<%= content[i].pic %>" alt="" /></a>' +
            '<% if( content[i].title ) { %><p><%= content[i].title %></p><% } %>' +
            '</div><%}%>' +
            '</div>' +
            '<%if(needSliderButton){%><span class="cui-pic-slider-pre"></span><span class="cui-pic-slider-next"></span><%}%>' +
            '<%if(needSliderDots){%><p class="cui-pic-slider-dots"><%for(i=0;i<len;i++){%>' +
            '<b></b>' +
            '<%}%></p><%}%>',
        getEl:function(){
            return this._options.el||this._options.container;
        },
        _create: function() {
            var me = this,
                $el = me.getEl(),
                opts = me._options;
            if(opts.scan){
                /*for(var p in this.options){}*/
                if($el.attr("data-loop")){
                    opts.loop=$el.attr("data-loop")==="true";
                }
                if($el.attr("data-index")){
                    opts.index= parseInt($el.attr("data-index"));
                }
                if($el.attr("data-speed")){
                    opts.speed= parseInt($el.attr("data-speed"));
                }
                if($el.attr("data-needSliderDots")){
                    opts.needSliderDots= $el.attr("data-needSliderDots")==="true";
                }
            }
            me.index = opts.index;
            // 初始dom结构
            me._initDom( $el, opts );
            me._parseMember();
            // 更新width
            me._initWidth( $el, me.index );
            me._loadImg();
            me._moveDots();
            me._bindEvent();

        },

        _initDom: function( $el, opts ) {

            // 非扫描创建
            if ( !opts.scan ) {
                $el.addClass("cui-pic-slider-container");
                this._createItems( $el, opts );
            }else{
                if(opts.needSliderDots){
                    var html=['<p class="cui-pic-slider-dots">'], i,
                        items=opts.container.find(".cui-pic-slider-group").children();
                    for(i=0;i<items.size();i++){
                        if(i===opts.index){
                            html.push('<b class="cui-state-active"></b>');
                        }
                        else{
                            html.push('<b></b>');
                        }
                    }
                    html.push('</p>');
                    $el.append(html.join(''));
                }

            }


        },

        _parseMember:function(){
            var opts =this._options,
                viewNum = opts.viewNum || 1,
                items,
                container=this.getEl();
            this._viewGroup = container.find(".cui-pic-slider-group");
            if(opts.needSliderDots)
                this._sliderDots = container.find(".cui-pic-slider-dots");
            if(opts.needSliderButton)
                this._sliderButton = {
                    next:container.find(".cui-pic-slider-next"),
                    prev:container.find(".cui-pic-slider-pre")
                };
            items= this._viewGroup.children();
            // 检测是否构成循环条件
            if (items.size() < viewNum + 1 ) {
                opts.loop = false;
            }

            // 如果节点少了，需要复制几份
            while ( opts.loop && this._viewGroup.children().size() < 3 * viewNum ) {
                this._viewGroup.append( items.eq(0).clone());
            }

            this.length = this._viewGroup.children().length;
            this._items = this._viewGroup.children();
        },
        // 根据items里面的数据挨个render插入到container中
        _createItems: function( container, opts ) {
            var fn =$.template.compile(this.template),str=fn(opts);
            container.html(str);
        },

        _initWidth: function( $el, index, force ) {
            var me = this,
                width;
            // width没有变化不需要重排
            if ( !force && (width = $el.width()) === me.width ) {
                return;
            }
            me.width = width;
            me._arrange( width, index );
           /* me.height = $el.height();*/
            me.trigger( 'width.change' );
        },

        // 重排items
        /**
         *
         * @param width
         * @param index当前显示项
         * @private
         */
        _arrange: function( width, index ) {
            this._slidePos = new Array( this.length );
            var self =this;
            this._items.each(function(i,item){
                $(item).css({width:width,left:(i * -width)}).attr("data-index",i);
                self._move( i, i < index ? -width : i > index ? width : 0, 0 );
            });
            this._viewGroup.css( 'width', width * this.length );
        },

        _move: function( index, dist, speed, immediate ) {
            if(index<0){return;}
            var slidePos = this._slidePos,
                items = this._items;

            if ( slidePos[ index ] === dist || !items.get(index) ) {
                return;
            }

            this._translate( index, dist, speed );
            slidePos[ index ] = dist;    // 记录目标位置

            // 强制一个reflow
            immediate && items.get(index).clientLeft;
        },

        _translate: function( index, dist, speed ) {
            var slide = this._items.get(index),
                style = slide && slide.style;

            if ( !style ) {
                return false;
            }

            style.cssText += cssPrefix + 'transition-duration:' + speed +
                'ms;' + cssPrefix + 'transform: translate(' +
                dist + 'px, 0);translateZ(0);';
        },

        _circle: function( index, arr ) {
            var len;

            arr = arr || this._items;
            len = arr.length;

            return (index % len + len) % arr.length;
        },
        _loadImg:function(){
            var item=this._items.eq(this.index);
            item.find("img[lazyload]").each(function(){
                this.setAttribute("src",this.getAttribute("lazyload"));
                this.removeAttribute("lazyload");
            });
        },
        _moveDots:function(){
            var to =this.index;
            if(this._sliderDots){
                this._sliderDots.find("b").removeClass("cui-state-active").eq(to).addClass("cui-state-active");
            }
        },
        _bindEvent:function(){
            var me =this,start,moved,delta;
            $.bind(me,"slide",function(){
                me._loadImg();
                me._moveDots();
            });

            // 转屏事件检测
            $( window ).on( 'ortchange', function() {
                me._initWidth( me._options.el, me.index );
            } );

            me.getEl().on("touchstart",function(e){
                if ( e.touches.length > 1 ) {
                    return false;
                }
                var touche  = e.touches[0],
                    opts = me._options,
                    num  = opts.viewNum || 1;
                start = {
                    x: touche.pageX,
                    y: touche.pageY,
                    time: +new Date()
                };
                delta={};
                moved=false;
                me._move( opts.loop ? me._circle( me.index - num ) :
                    me.index - num, -me.width, 0, true );

                me._move( opts.loop ? me._circle( me.index + num ) :
                    me.index + num, me.width, 0, true );


            }).on("touchmove",function(e){
                    // 多指或缩放不处理
                    if ( e.touches.length > 1 || e.scale &&
                        e.scale !== 1 ) {
                        return false;
                    }
                    var opts = me._options,
                        viewNum = opts.viewNum || 1,
                        touche = e.touches[ 0 ],
                        index = me.index,
                        i,
                        len,
                        pos,
                        isScrolling,
                        slidePos;
                    delta.x = touche.pageX - start.x;
                    delta.y = touche.pageY - start.y;
                    isScrolling = Math.abs( delta.x ) < Math.abs( delta.y );
                    if(!isScrolling){
                        e.preventDefault();
                        if ( !opts.loop ) {
                            // 如果左边已经到头
                            delta.x /= (!index && delta.x > 0 ||
                                // 如果右边到头
                                index === me._items.length - 1 &&
                                    delta.x < 0) ?
                                // 则来一定的减速
                                (Math.abs( delta.x ) / me.width + 1) : 1;
                        }
                        slidePos = me._slidePos;
                        for ( i = index - viewNum, len = index + 2 * viewNum;
                              i < len; i++ ) {
                            pos = opts.loop ? me._circle( i ) : i;
                            me._translate( pos, delta.x + slidePos[ pos ], 0 );
                        }
                        moved = true;
                    }

                }).on("touchend",function(e){
                    if ( !moved ) {
                        return;
                    }
                    var  opts = me._options,
                        viewNum = opts.viewNum || 1,
                        index = me.index,
                        slidePos = me._slidePos,
                        duration = +new Date() - start.time,
                        absDeltaX = Math.abs( delta.x ),

                    // 是否滑出边界
                        isPastBounds = !opts.loop && (!index && delta.x > 0 ||
                            index === slidePos.length - viewNum && delta.x < 0),

                    // -1 向右 1 向左
                        dir = delta.x > 0 ? 1 : -1,
                        speed,
                        diff,
                        i,
                        len,
                        pos;
                    if ( duration < 250 ) {
                        // 如果滑动速度比较快，偏移量跟根据速度来算
                        speed = absDeltaX / duration;
                        diff = Math.min( Math.round( speed * viewNum * 1.2 ),
                            viewNum );
                    } else {
                        diff = Math.round( absDeltaX / (me.perWidth || me.width) );
                    }
                    if ( diff && !isPastBounds ) {
                        me._slide( index, diff, dir, me.width, opts.speed,
                            opts, true );

                        /*                             // 在以下情况，需要多移动一张
                         if ( viewNum > 1 && duration >= 250 &&
                         Math.ceil( absDeltaX / me.perWidth ) !== diff ) {

                         me.index < index ? me._move( me.index - 1, -me.perWidth,
                         opts.speed ) : me._move( me.index + viewNum,
                         me.width, opts.speed );
                         }*/
                    } else {

                        // 滑回去
                        for ( i = index - viewNum, len = index + 2 * viewNum;
                              i < len; i++ ) {
                            pos = opts.loop ? me._circle( i ) : i;
                            me._translate( pos, slidePos[ pos ],
                                opts.speed );
                        }
                    }

                });

        },
        _tansitionEnd: function( e ) {

            // ~~用来类型转换，等价于parseInt( str, 10 );
            if ( ~~e.target.getAttribute( 'data-index' ) !== this.index ) {
                return;
            }

            this.trigger( 'slideend', this.index );


        },

        _slide: function( from, diff, dir, width, speed, opts ) {
            var me = this,
                to;
            to = me._circle( from - dir * diff );

            // 如果不是loop模式，以实际位置的方向为准
            if ( !opts.loop ) {
                dir = Math.abs( from - to ) / (from - to);
            }

            // 调整初始位置，如果已经在位置上不会重复处理
            this._move( to, -dir * width, 0, true );

            this._move( from, width * dir, speed );
            this._move( to, 0, speed );

            this.index = to;
            return this.trigger( 'slide', to, from );
        },

        /**
         * 切换到第几个slide
         * @method slideTo
         * @chainable
         * @param {Number} to 目标slide的序号
         * @param {Number} [speed] 切换的速度
         * @return {self} 返回本身
         */
        slideTo: function( to, speed ) {
            if ( this.index === to || this.index === this._circle( to ) ) {
                return this;
            }

            var opts = this._options,
                index = this.index,
                diff = Math.abs( index - to ),

            // 1向左，-1向右
                dir = diff / (index - to),
                width = this.width;

            speed = speed || opts.speed;

            return this._slide( index, diff, dir, width, speed, opts );
        },

        /**
         * 切换到上一个slide
         * @method prev
         * @chainable
         * @return {self} 返回本身
         */
        prev: function() {

            if ( this._options.loop || this.index > 0 ) {
                this.slideTo( this.index - 1 );
            }

            return this;
        },

        /**
         * 切换到下一个slide
         * @method next
         * @chainable
         * @return {self} 返回本身
         */
        next: function() {

            if ( this._options.loop || this.index + 1 < this.length ) {
                this.slideTo( this.index + 1 );
            }

            return this;
        },

        /**
         * 返回当前显示的第几个slide
         * @method getIndex
         * @chainable
         * @return {Number} 当前的silde序号
         */
        getIndex: function() {
            return this.index;
        },
        trigger:function(evt,arg1,arg2){
            $.trigger(this,evt,[arg1,arg2]);
        },
        /**
         * 销毁组件
         * @method destroy
         */
        destroy: function() {
            this._container.off( this.eventNs );
            $( window ).off( 'ortchange' + this.eventNs );
            return this.$super( 'destroy' );
        }

    };

    $.fn.picSlider =function(opts){
        opts.el=this.eq(0);
        return new picSlider(opts);
    };

    $.component.picSlider = picSlider;

})(af);