(function ($){
    var win =window;
    var  ruleMap ={
            /**
             * 必需验证
             * @param {*} value
             * @returns {boolean}
             */
            required: function(value) {
                switch ( $.type(value) ) {
                    case "string":
                    case "array":
                        return value.length > 0;
                    case "null":
                    case "undefined":
                        return false;
                    default:
                        return true;
                }
            },
            /**
             * email
             * @param value
             * @returns {boolean}
             */
            email: function (value) {
                return typeof value === "string" &&
                    /^[a-zA-Z0-9.!#$%&'*+\/=?^_`{|}~-]+@[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?)*$/.test(value);
            },
            /**
             * url http(s), ftp
             * @param value
             * @returns {boolean}
             */
            url: function (value) {
                // contributed by Scott Gonzalez: http://projects.scottsplayground.com/iri/
                return typeof value === "string" && /^(https?|s?ftp):\/\/(((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:)*@)?(((\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5]))|((([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.)+(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.?)(:\d*)?)(\/((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)+(\/(([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)*)*)?)?(\?((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)|[\uE000-\uF8FF]|\/|\?)*)?(#((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)|\/|\?)*)?$/i.test(value);
            },
            /**
             * 数字，支持加减号，小数点
             * @param value
             * @returns {boolean}
             */
            number: function (value) {
                return /number|string/.test(typeof value) && /^(\+|-)?(?:\d+|\d{1,3}(?:,\d{3})+)?(?:\.\d+)?$/.test(value);
            },
            /**
             * 纯数字，只能数字
             * @param value
             * @returns {boolean}
             */
            digits: function(value) {
                return /number|string/.test(typeof value) && /^\d+$/.test(value);
            },
            /**
             * 整数
             * @param value
             * @returns {boolean}
             */
            integer: function (value) {
                return /number|string/.test(typeof value) && /^(\+|-)?\d+$/.test(value);
            },
            /**
             * 是否是浮点数，支持科学计数与整数
             * @param value
             * @returns {boolean}
             */
            float: function (value) {
                return /number|string/.test(typeof value) && /^(?:-?(?:[0-9]+))?(?:\.[0-9]*)?(?:[eE][\+\-]?(?:[0-9]+))?$/.test(value);
            },
            /**
             * 是否是IP
             * @param value
             * @returns {boolean}
             */
            ip: function (value) {
                return typeof value === "string" && (
                    /^(\d?\d?\d)\.(\d?\d?\d)\.(\d?\d?\d)\.(\d?\d?\d)$/.test(value) ||
                        /::|^::1|^([a-fA-F0-9]{1,4}::?){1,7}([a-fA-F0-9]{1,4})/.test(value)
                    );
            },
            /**
             * 是否是JSON字符串
             * @param value
             * @returns {boolean}
             */
            JSONStr: function (value) {
                var JSON = win.JSON,
                    result;

                try {
                    if (JSON && JSON.parse) {
                        result = JSON.parse(value);
                    } else {
                        win.eval("result=" + value + "");
                    }
                    return typeof result === "object" && result !== null;
                } catch (e) {
                    return false;
                }
            },
            /**
             * 最小字符个数
             * @param value
             * @param min
             * @returns {boolean}
             */
            minLength: function (value, min) {
                switch ( $.type(value) ) {
                    case "number":
                        value = value.toString();
                    case "string":
                    case "array":
                        return value.length >= min;
                    default:
                        return false;
                }
            },
            /**
             * 最大字符个数
             * @param value
             * @param max
             * @returns {boolean}
             */
            maxLength: function (value, max) {
                switch ( $.type(value) ) {
                    case "number":
                        value = value.toString();
                    case "string":
                    case "array":
                        return value.length <= max;
                    default:
                        return false;
                }
            },
            /**
             * 字符范围
             * @param value
             * @param min
             * @param max
             * @returns {boolean}
             */
            rangeLength: function (value, min, max) {
                return this.minLength(value, min) &&
                    this.maxLength(value, max);
            },
            /**
             * 最小字节数
             * @param value
             * @param min
             * @returns {boolean}
             */
            minBytes: function (value, min) {
                switch (typeof value) {
                    case "string":
                        value = value.replace(/[^\x00-\xff]/g, "..");
                    case "number":
                        return this.minLength(value, min);
                    default :
                        return false;
                }
            },
            /**
             * 最大字节数
             * @param value
             * @param max
             * @returns {boolean}
             */
            maxBytes: function (value, max) {
                switch (typeof value) {
                    case "string":
                        value = value.replace(/[^\x00-\xff]/g, "..");
                    case "number":
                        return this.maxLength(value, max);
                    default :
                        return false;
                }
            },
            /**
             * 字节数范围
             * @param value
             * @param min
             * @param max
             * @returns {boolean}
             */
            rangeBytes: function (value, min, max) {
                switch (typeof value) {
                    case "string":
                        value = value.replace(/[^\x00-\xff]/g, "..");
                    case "number":
                        return this.minLength(value, min) &&
                            this.maxLength(value, max);
                    default :
                        return false;
                }
            },
            /**
             * 最小值
             * @param value
             * @param min
             * @returns {boolean}
             */
            min: function (value, min) {
                if (typeof value === "string" && this.number(value)) {
                    value = Number(value)
                }
                try {
                    return value >= min;
                } catch (e) {
                    return false;
                }
            },
            /**
             * 最大值
             * @param value
             * @param max
             * @returns {boolean}
             */
            max: function (value, max) {
                if (typeof value === "string" && this.number(value)) {
                    value = Number(value)
                }
                try {
                    return value <= max;
                } catch (e) {
                    return false;
                }
            },
            /**
             * 值范围
             * @param value
             * @param min
             * @param max
             * @returns {boolean}
             */
            range: function (value, min, max) {
                return this.min(value, min) && this.max(value, max);
            },
            /**
             * 相等比较
             * @param value
             * @param eqValue
             * @returns {boolean}
             */
            equal: function (value, eqValue) {
                return value === eqValue;
            },
            /**
             * 相等
             * @param value
             * @param ele
             * @returns {boolean}
             */
            equalTo: function (value, ele) {
                if (typeof ele === "string") {
                    ele = $(ele).eq(0);
                }
                switch (true) {
                    case !!ele.getValue:
                        return value === ele.getValue();
                    case !!ele.val:
                        return value === ele.val();
                    case !!ele.getAttribute:
                        return value === ele.value;
                    default:
                        return false;
                }
            },
            /**
             * value包含在list中
             * @param value
             * @param list
             * @returns {boolean}
             */
            withIn: function (value, list) {
                var i, j, item;
                if ($.type(value) !== "array") {
                    value = [value];
                }
                valLoop: for (i = value.length; i--;) {
                    item = value[i];
                    for (j = list.length; j--;) {
                        if (item === list[j]) {
                            continue valLoop;
                        }
                    }
                    return false;
                }
                return true;
            },
            /**
             * value不包含在list中
             * @param value
             * @param list
             * @returns {boolean}
             */
            withOut: function (value, list) {
                var i, j, item;
                if ($.type(value) !== "array") {
                    value = [value];
                }
                for (i = value.length; i--;) {
                    item = value[i];
                    for (j = list.length; j--;) {
                        if (item === list[j]) {
                            return false;
                        }
                    }
                }
                return true;
            },
            /**
             * 自定义匹配,可以是函数、正则
             * @param value
             * @param pattern
             * @returns {*}
             */
            custom: function (value, pattern) {
                if (typeof pattern === "function") {
                    return !!pattern(value);
                } else {
                    try {
                        return pattern.test(value);
                    } catch(e) {
                        return false;
                    }
                }
            }
        };

    var  validate= function() {
          this.validate={};
    };

    validate.prototype = {
        /**
         *
         * @param el dom
         * @param rules  {type:"required",msg:"XXX不能为空",arg:[]} | [{},{}]
         * @returns {*}
         */
        addRule: function (el,rules) {
            var validate = this.validate,
                id= el.id,type=$.type(rules);
             if(!id){
                id ="cui-form-id-"+(90*Math.random()+10).toString().replace(".","");
                el.id = id;
             }
            if(validate[id]){
                if(type==="array"){
                    validate[id] = validate[id].concat(rules);
                }else if(type==="object"){
                    validate[id].push(rules);
                }
            } else{
                if(type==="array"){
                    validate[id] = rules;
                }else if(type==="object"){
                    validate[id] =[rules];
                }
            }
            return this;
        },
        /**
         *
         * @param dom
         * @param types  "required" | ["required","min"]
         * @returns {*}
         */
        removeRule: function (dom,types) {
            var validate = this.validate,
                id       = dom.id,
                i,
                newarr   = [],
                valid    = validate[id];
            if (valid) {
                if (typeof types === "string") {
                    types = [types];
                }
                if ($.type(types) === "array") {
                    for(var j= 0,len=valid.length;j<len;j++){
                        var flag =false;
                        for(i=types.length-1;i>=0;i--){
                             if(valid[j].type===types[i]){
                                 flag =true;break;
                             }
                        }
                        if(!flag){
                            newarr.push(valid[j]);
                        }
                    }

                    this.validate[id] =  newarr;

                }
            }
            return this;
        },
        /**
         * 验证失效
         */
        disable: function () {
            this.flag =false;
            return this;
        },
        enable:function(){
            this.flag =true;
        },
        /**
         * 进行验证
         * @returns {*} 成功返回true，失败返回失败信息
         */
        doValidate: function (id) {
            if(this.flag===false){
                return true;
            }
            var p,
                validate = this.validate,
                i,len,
                value,
                res;
            var validateValue = function(value,rules){
                var type;
                for(i=0,len=rules.length;i<len;i++){
                    type =rules[i].type;
                    if(ruleMap[type]){
                        var res =  ruleMap[type].apply(ruleMap,[value].concat(rules[i].arg||[]));
                        if(res===false){
                            return rules[i].msg;
                        }
                    }
                }
                return true;
            };
            if(id&&validate[id]){
                value = document.getElementById(id).value;
                res   = validateValue(value,validate[id]);
                if(res!==true){
                    return {
                        id:id,
                        msg:res
                    }
                }
                return true;
            }
                for (p in validate) {
                   if(validate.hasOwnProperty(p)){
                        value = document.getElementById(p).value;
                        res   = validateValue(value,validate[p]);
                        if(res!==true){
                           return {
                               id:p,
                               msg:res
                           }
                        }

                   }
                }
            return true;
        }

    };

    $.validate = function(){
       return new validate();
    };
})(window.af);